%% Description

% This script is used to visualize the characteristic load profiles for
% different load types and evaluate the typical load profile parameters.

%% Initialization

% Prepare the environment

clear variables
close all
clc

% Load the data from Excel

load_data = readtable("Load profiles.xlsx") ;

% Generate the Date/Time column (from 01-Jan-2023 00:00 to 31-Dec-2023 23:00)

date_time = datetime(2023,1,1,0,0,0) : hours(1) : datetime(2023,12,31,23,0,0) ;
load_data.DateTime = date_time' ;

% Add Month and Hour columns to the table

load_data.Month = month(load_data.DateTime) ;
load_data.Hour = hour(load_data.DateTime) ;

% Determine the season for each row

load_data.Season = string(arrayfun(@(m) getSeason(m),load_data.Month,'UniformOutput',false)) ;

% List of load types (assumes load types are in columns 1 to 7)

load_types = load_data.Properties.VariableNames(1:7) ;

%% Determine and visualize the seasonal load profiles

% Initialize a structure to store seasonal profiles

seasonal_profiles = struct() ;

% Calculate the average daily load profile for each load type by season

unique_seasons = ["Winter", "Spring", "Summer", "Autumn"] ;

for i = 1 : length(load_types)
    
    % Extract the load type

    load_type = load_types{i} ;
    
    % Create an empty matrix to store seasonal profiles

    seasonal_profile = zeros(24, length(unique_seasons)) ;
    
    % Loop through each season and calculate the mean for each hour

    for j = 1 : length(unique_seasons)

        season = unique_seasons(j) ;
        
        % Extract rows for the current season and load type

        season_data = load_data(strcmp(load_data.Season, season),:) ;
        
        % Calculate the average load for each hour of the day

        for h = 0 : 23 
         
            hourly_data = season_data.(load_type)(season_data.Hour == h) ;
            seasonal_profile(h+1, j) = mean(hourly_data) ;
        
        end
    
    end
    
    % Store the seasonal profile in the structure
    
    seasonal_profiles.(load_type) = seasonal_profile ;

end

% Plotting the seasonal load profiles for each load type

for i = 1 : length(load_types)

    load_type = load_types{i} ;
    profile = seasonal_profiles.(load_type) ;
    
    figure
    hold on 

    for j = 1:length(unique_seasons)

        plot(0:23, profile(:,j),'DisplayName',unique_seasons(j)) ;

    end
    
    title(['Characteristic Daily Load Profile for ' load_type]) ;
    xlabel('Time [h]') ;
    ylabel('Average hourly load [kW]') ;
    legend('show','Location','best') ;
    grid on ;

end

%% Determine the characteristic load parameters

% Initialize arrays to store results for each load type

minimumLoadArray = zeros(length(load_types),1) ;
maximumLoadArray = zeros(length(load_types),1) ;
averageLoadArray = zeros(length(load_types),1) ;
totalConsumptionArray = zeros(length(load_types),1) ;
loadFactorArray = zeros(length(load_types),1) ;
fullLoadHoursArray = zeros(length(load_types),1) ;

% Loop through each load type and calculate the characteristic parameters

for i = 1 : length(load_types)

    % Extract the load data for the current load type
    
    load_type = load_types{i} ;
    load = load_data.(load_type) ;
    
    % Call the evaluateLoadParameters function
    
    [minimumLoad,maximumLoad,averageLoad,totalConsumption,loadFactor,fullLoadHours] = evaluateLoadParameters(load) ;
    
    % Store the results in the arrays

    minimumLoadArray(i) = minimumLoad ;
    maximumLoadArray(i) = maximumLoad ;
    averageLoadArray(i) = averageLoad ;
    totalConsumptionArray(i) = totalConsumption ;
    loadFactorArray(i) = loadFactor ;
    fullLoadHoursArray(i) = fullLoadHours ;

end

% Create a table with the results

resultsTable = table(load_types', minimumLoadArray, maximumLoadArray, averageLoadArray, totalConsumptionArray, loadFactorArray, fullLoadHoursArray, ...
    'VariableNames', {'LoadType','MinimumLoad','MaximumLoad','AverageLoad','TotalConsumption','LoadFactor','FullLoadHours'}) ;

% Display the table

disp(resultsTable)

%% Auxiliary functions

% Function to determine the season based on the month

function season = getSeason(month)

    if ismember(month,[12,1,2])

        season = "Winter" ;

    elseif ismember(month,[3,4,5])

        season = "Spring" ;

    elseif ismember(month,[6,7,8])

        season = "Summer" ;
    else

        season = "Autumn" ;

    end

end

% Function to determine the characteristic load parameters

function [minimumLoad,maximumLoad,averageLoad,totalConsumption,loadFactor,fullLoadHours] = evaluateLoadParameters(load)

minimumLoad = min(load) ;
maximumLoad = max(load) ;
averageLoad = mean(load) ; 
totalConsumption = sum(load) ; 
loadFactor = totalConsumption/(8760*maximumLoad) ; 
fullLoadHours = totalConsumption/maximumLoad ; 

end